unit formPTChart;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Data.DB,
  VCLTee.TeEngine, VCLTee.TeeURL, VCLTee.TeeSeriesTextEd, VCLTee.Series,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.DBChart, VCLTee.TeeTools,
  Vcl.StdCtrls, Vcl.buttons, VCLTee.TeeMagnifyTool, VCLTee.TeeScrollPagerTool, System.StrUtils,
  system.Math;

type
  TfrmPTChart = class(TForm)
    DBChart1: TDBChart;
    Series1: TLineSeries;
    Series2: TLineSeries;
    SeriesTextSource1: TSeriesTextSource;
    SeriesTextSource2: TSeriesTextSource;
    Series3: TLineSeries;
    SeriesTextSource3: TSeriesTextSource;
    SeriesTextSource4: TSeriesTextSource;
    CheckBox1: TCheckBox;
    RadioGroup1: TRadioGroup;
    ChartTool1: TAxisScrollTool;
    ChartTool2: TAxisScrollTool;
    ChartTool3: TAxisScrollTool;
    ChartTool4: TAxisScrollTool;
    Panel1: TPanel;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure DBChart1BeforeDrawAxes(Sender: TObject);
    procedure DBChart1GetAxisLabel(Sender: TChartAxis;
      Series: TChartSeries; ValueIndex: Integer; var LabelText: string);
    procedure CheckBox1Click(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);

  private
    procedure ReadToArrays;
    function VDEL(aMD: double; VE: array of double): double;
    procedure SyncTopAxis;
    procedure SyncVertAxes;
    procedure SetRightAxisTitle;
    procedure ApplyChartLogLabels;
    procedure CalculateLabels(AAxis: TChartAxis; AInc: integer);
    procedure GenerateLabelArray(AInc: integer);

    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmPTChart: TfrmPTChart;
  MD: array of double;
  VD: array of double;
  EL: array of double;
  AVal: array of double;
  bFirstTime: boolean;

implementation

{$R *.dfm}

// ========== TfrmPTChart.FormShow =============================================
procedure TfrmPTChart.FormCreate(Sender: TObject);
begin
  FormatSettings.DecimalSeparator:='.';
  FormatSettings.ThousandSeparator:=',';
end;

procedure TfrmPTChart.FormShow(Sender: TObject);
var
  i: integer;
  LStrList: TStringList;
begin
// Load Pressure vs Depth (Left Axis)
  Series1.DataSource := SeriesTextSource1;
  Series1.VertAxis := aBothVertAxis;

  with SeriesTextSource1 do
  begin
    FileName := '..\..\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',3);
    AddField('Y',1);
    Load;
  end;

// Load WHP vs Depth (Left Axis)
  Series2.DataSource := SeriesTextSource2;
  Series2.VertAxis := aBothVertAxis;

  with SeriesTextSource2 do
  begin
    FileName := '..\..\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',4);
    AddField('Y',1);
    Load;
  end;

// Load Temperature vs Depth (Left Axis)
  Series3.DataSource := SeriesTextSource3;
  Series3.VertAxis := aBothVertAxis;

  with SeriesTextSource3 do
  begin
    FileName := '..\..\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',2);
    AddField('Y',1);
    Load;
  end;

  ReadToArrays;

  SyncVertAxes;

  SetRightAxisTitle;

end;

// ========== TfrmPTChart.RadioGroup1Click =====================================
procedure TfrmPTChart.RadioGroup1Click(Sender: TObject);
begin
  SetRightAxisTitle;
  ApplyChartLogLabels;
  DBChart1.Repaint;
end;

// ========== TfrmPTChart.ReadToArrays =========================================
// Read Measured Depth/Vertical Depth/Elevation for well
procedure TfrmPTChart.ReadToArrays;
var
  Lsl: TStringList;
  i: integer;
  s: string;

begin
  Lsl := TStringList.Create;
  try
    Lsl.LoadFromFile('..\..\AT-203_Deviation.csv');

    SetLength(MD,Lsl.Count);
    SetLength(VD,Lsl.Count);
    SetLength(EL,Lsl.Count);

    for i := 1 to Lsl.Count - 1 do
    begin
      s := Lsl[i];
      begin
        MD[i-1] := StrToFloat(Copy(s,1,Pos(',',s)-1));
        s := Copy(s,Pos(',',s) + 1,Length(s));
        VD[i-1] := StrToFloat(Copy(s,1,Pos(',',s)-1));
        EL[i-1] := StrToFloat(Copy(s,Pos(',',s) + 1,Length(s)));
      end;
    end;
  finally
    Lsl.Free;
  end;
end;

// ========== TfrmPTChart.VDEL =================================================
// Calculate Vertical Depth or Elevation for any specified depth
// Use VD or EL array as appropriate
function TfrmPTChart.VDEL(aMD: double; VE: array of double): double;
var
  i,j: Integer;

begin
  if aMD < MD[0] then
    j := 0
  else
  if aMD > MD[Length(MD)-2] then
    j := Length(MD) - 3
  else
  for i := 1 to Length(MD) - 2 do
  begin
    if aMD < MD[i] then
    begin
      j := i - 1;
      break;
    end;
  end;

  result := (aMD - MD[j])/(MD[j+1] - MD[j])*(VE[j+1]-VE[j]) + VE[j];

end;

// ========== TfrmPTChart.CheckBox1Click =======================================
// Doesn't work very well
procedure TfrmPTChart.CheckBox1Click(Sender: TObject);
begin
  if CheckBox1.Checked then
  begin
  end
  else
  begin
    DBChart1.Axes.Top.Items.Automatic:=True;
  end;
  DBChart1.Repaint;
end;

// ========== TfrmPTChart.CheckBox2Click =======================================
procedure TfrmPTChart.CheckBox2Click(Sender: TObject);
begin
  bFirstTime := True;
  DBChart1.Axes.Left.Logarithmic := CheckBox2.Checked;
  //ApplyChartLogLabels;
  if DBChart1.Axes.Left.Logarithmic then
    DBChart1.Axes.Left.Axis.Color := clLime
  else
    DBChart1.Axes.Left.Axis.Color := clBlack;

  DBChart1.Axes.Right.Logarithmic := CheckBox2.Checked;
  {if not DBChart1.Axes.Right.Logarithmic then
  begin
    DBChart1.Axes.Right.Items.Automatic := true;
    DBChart1.Axes.Right.Automatic := true;
    DBChart1.Axes.Right.Labels := true;
  end;}
  if DBChart1.Axes.Right.Logarithmic then
    DBChart1.Axes.Right.Axis.Color := clLime
  else
    DBChart1.Axes.Right.Axis.Color := clBlack;

  ApplyChartLogLabels;
end;

// ========== TfrmPTChart.CheckBox3Click =======================================
procedure TfrmPTChart.CheckBox3Click(Sender: TObject);
begin
  DBChart1.Axes.Top.Logarithmic := CheckBox3.Checked;
  ApplyChartLogLabels;
  if DBChart1.Axes.Top.Logarithmic then
    DBChart1.Axes.Top.Axis.Color := clLime
  else
    DBChart1.Axes.Top.Axis.Color := clBlack;
end;

// ========== TfrmPTChart.CheckBox4Click =======================================
procedure TfrmPTChart.CheckBox4Click(Sender: TObject);
begin
  DBChart1.Axes.Bottom.Logarithmic := CheckBox4.Checked;
  ApplyChartLogLabels;
  if DBChart1.Axes.Bottom.Logarithmic then
    DBChart1.Axes.Bottom.Axis.Color := clLime
  else
    DBChart1.Axes.Bottom.Axis.Color := clBlack;
end;

// ========== TfrmPTChart.DBChart1BeforeDrawAxes ===============================
// Now works on zoom, scroll etc
procedure TfrmPTChart.DBChart1BeforeDrawAxes(Sender: TObject);
var
  iInc: integer;

begin
  if CheckBox1.Checked then
    SyncTopAxis;
end;

// ========== TfrmPTChart.DBChart1GetAxisLabel =================================
procedure TfrmPTChart.DBChart1GetAxisLabel(Sender: TChartAxis;
  Series: TChartSeries; ValueIndex: Integer; var LabelText: string);
var
  tmp: Double;
  iInc: integer;

begin
  if Sender = DBChart1.Axes.Right then
  begin
//    if {Sender.Logarithmic and} bFirstTime then
//    begin
//      iInc := 3;
//      SetLength(AVal, iInc);
//      CalculateLabels(DBChart1.Axes.Left, iInc);
//      bFirstTime := False;
//    end;
    tmp:=StrToFloatDef(ReplaceStr(LabelText, FormatSettings.ThousandSeparator,''), 0);
    if (RadioGroup1.ItemIndex = 1) then // measured depth
    begin
//    tmp:=Round(tmp);
    end
    else
    if (RadioGroup1.ItemIndex = 2) then // vertical depth
    begin
      tmp:=Round(VDEL(tmp, VD));
    end
    else
    if (RadioGroup1.ItemIndex = 3) then // elevation
    begin
      tmp:=Round(VDEL(tmp, EL));
    end;
    LabelText:=FormatFloat(Sender.AxisValuesFormat, tmp);
  end;
end;

// ========== TfrmPTChart.SyncVertAxes =========================================
procedure TfrmPTChart.SyncVertAxes;
begin
  DBChart1.RightAxis.AutomaticMinimum := DBChart1.LeftAxis.AutomaticMinimum;
  DBChart1.RightAxis.AutomaticMaximum := DBChart1.LeftAxis.AutomaticMaximum;
  DBChart1.RightAxis.Automatic := DBChart1.LeftAxis.Automatic;
  DBChart1.RightAxis.Minimum := DBChart1.LeftAxis.Minimum;
  DBChart1.RightAxis.Maximum := DBChart1.LeftAxis.Maximum;
  DBChart1.RightAxis.Increment := DBChart1.LeftAxis.Increment;
  DBChart1.RightAxis.Inverted := DBChart1.LeftAxis.Inverted;
  DBChart1.RightAxis.Logarithmic := DBChart1.LeftAxis.Logarithmic;
end;

// ========== TfrmPTChart.SyncTopAxis ==========================================
// Uses bottom axis increments to locate top axis labels
procedure TfrmPTChart.SyncTopAxis;
var
  i, iMin, iMax, Factor: integer;
  d: double;
  Tval, TPos, TMin, TMax, TDiff, BMin, BMax, BDiff, BInc: double;

begin
  Factor := 1;
  DBChart1.Axes.Top.Items.Clear;
  TMin := DBChart1.Axes.Top.Minimum;
  TMax := DBChart1.Axes.Top.Maximum;
  TDiff := TMax - Tmin;
  BMin := DBChart1.Axes.Bottom.Minimum;
  BMax := DBChart1.Axes.Bottom.Maximum;
  BDiff := BMax - BMin;
  BInc := DBChart1.Axes.Bottom.CalcIncrement;

  if (BInc < 0.01)  then
    Factor := 1000
  else if (BInc < 0.1) then
    Factor := 100
  else if (BInc < 1) then
    Factor := 10;

  iMin := Round(BMin/BInc);
  iMax := Round(BMax/BInc);

  for i := iMin to iMax do
  begin
    Tval := Round(((i * BInc - BMin)/BDiff*TDiff + TMin)*Factor)/Factor;
    TPos := (i * BInc - Bmin)/BDiff*TDiff + TMin;
    DBChart1.Axes.Top.Items.Add(TPos, FloatToStr(Tval));  // adding custom label
  end;

end;

// ========== TfrmPTChart.ApplyChartLogLabels ==================================
procedure TfrmPTChart.ApplyChartLogLabels;
var
  iInc: integer;

begin
  iInc := 3;
  SetLength(AVal, iInc);
  CalculateLabels(DBChart1.Axes.Left, iInc);

  iInc := 3;
  SetLength(AVal, iInc);
  CalculateLabels(DBChart1.Axes.Right, iInc);

  iInc := 5;
  SetLength(AVal, iInc);
  CalculateLabels(DBChart1.Axes.Top, iInc);

  iInc := 7;
  SetLength(AVal, iInc);
  CalculateLabels(DBChart1.Axes.Bottom, iInc);
end;

// ========== TfrmPTChart.CalculateLabels ======================================
procedure TfrmPTChart.CalculateLabels(AAxis: TChartAxis; AInc: integer);
var
  i, j: integer;
  LMin, LMax, LVal: double;
  LLabel: String;
begin
  if AAxis.Logarithmic then
  begin
    AAxis.Items.Clear;
    if AAxis.Automatic then
      AAxis.CalcMinMax(LMin,LMax)
    else
    begin
    LMin := AAxis.Minimum;
    LMax := AAxis.Maximum;
    end;
    GenerateLabelArray(AInc);

    for i := -6 to 10 do
      for j := 0 to AInc - 1 do
      begin
        LVal := AVal[j] * System.Math.power(10, i);
        if (LVal > LMin) and (LVal < LMax) then
        begin
          if (AAxis = DBChart1.Axes.Right) and (RadioGroup1.ItemIndex in [2, 3]) then
          begin
            if (RadioGroup1.ItemIndex = 2) then // vertical depth
              LLabel:=FloatToStr(Round(VDEL(LVal, VD)))
            else if (RadioGroup1.ItemIndex = 3) then // elevation
              LLabel:=FloatToStr(Round(VDEL(LVal, EL)));
          end
          else
            LLabel:=FloatToStr(LVal);

          AAxis.Items.Add(LVal, LLabel);
        end;
      end;
  end
  else
  begin
    AAxis.Items.Automatic := true;
    AAxis.Automatic := true;
    AAxis.Labels := true;
  end;
end;


// ========== TfrmPTChart.GenerateLabelArray =================================
procedure TfrmPTChart.GenerateLabelArray(AInc: integer);

begin
  case AInc of
    1:
      begin
        AVal[0] := 1;
      end;
    2:
      begin
        AVal[0] := 1;
        AVal[1] := 3;
      end;
    3:
      begin
        AVal[0] := 1;
        AVal[1] := 2;
        AVal[2] := 5;
      end;
    4:
      begin
        AVal[0] := 1;
        AVal[1] := 2;
        AVal[2] := 3;
        AVal[3] := 5;
      end;
    5:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2.5;
        AVal[3] := 4;
        AVal[4] := 6;
      end;
    6:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2;
        AVal[3] := 3;
        AVal[4] := 5;
        AVal[5] := 7;
      end;
    7:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2;
        AVal[3] := 3;
        AVal[4] := 4;
        AVal[5] := 5;
        AVal[6] := 7;
      end;
    8:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2;
        AVal[3] := 3;
        AVal[4] := 4;
        AVal[5] := 5;
        AVal[6] := 6;
        AVal[7] := 8;
      end;
    9:
      begin
        AVal[0] := 1;
        AVal[1] := 2;
        AVal[2] := 3;
        AVal[3] := 4;
        AVal[4] := 5;
        AVal[5] := 6;
        AVal[6] := 7;
        AVal[7] := 8;
        AVal[8] := 9;
      end;
  else
    begin
      AVal[0] := 1;
      AVal[1] := 1.5;
      AVal[2] := 2;
      AVal[3] := 3;
      AVal[4] := 4;
      AVal[5] := 5;
      AVal[6] := 6;
      AVal[7] := 7;
      AVal[8] := 8;
      AVal[9] := 9;
    end;
  end;
end;

// ========== TfrmPTChart.SetRightAxisTitle ====================================
procedure TfrmPTChart.SetRightAxisTitle;
begin
  if (RadioGroup1.ItemIndex = 0) then
  begin
    DBChart1.RightAxis.Title.Text := '';
    DBChart1.OnGetAxisLabel := nil;
    Series1.VertAxis := aLeftAxis;
    Series2.VertAxis := aLeftAxis;
    Series3.VertAxis := aLeftAxis;
  end
  else
  if RadioGroup1.ItemIndex in [1, 2, 3] then
  begin
    Series1.VertAxis := aBothVertAxis;
    Series2.VertAxis := aBothVertAxis;
    Series3.VertAxis := aBothVertAxis;
    if (RadioGroup1.ItemIndex = 1) then
    begin
      DBChart1.RightAxis.Title.Text := 'Measured Depth (m)';
      DBChart1.OnGetAxisLabel := DBChart1GetAxisLabel;
    end
    else if (RadioGroup1.ItemIndex = 2) then
    begin
      DBChart1.RightAxis.Title.Text := 'Vertical Depth (m)';
      DBChart1.OnGetAxisLabel := DBChart1GetAxisLabel;
    end
    else if (RadioGroup1.ItemIndex = 3) then
    begin
      DBChart1.RightAxis.Title.Text := 'Elevation (m)';
      DBChart1.OnGetAxisLabel := DBChart1GetAxisLabel;
    end;
  end;
end;

end.
